import json

import requests

from netlenium.exceptions import *


class API(object):

    def __init__(self, endpoint="http://localhost:6410", authentication=None):
        """
        Public Constructor
        公共构造函数

        :param endpoint:
        :param authentication:
        """

        self.endpoint = endpoint
        self.authentication = authentication

    @staticmethod
    def parse_error(response):
        """
        Determines the error and throws the appropriate exception
        确定错误并引发相应的异常

        :param response:
        :return:
        """

        payload = json.loads(response.text)
        error_code = payload["ErrorCode"]
        if error_code == 1:
            raise NetleniumException(response.text, response.status_code, response.text)
        if error_code == 100:
            raise NetleniumException(payload["Error"], response.status_code, response.text)
        if error_code == 101:
            raise AttributeNotFound(payload["Message"], response.status_code, response.text)
        if error_code == 102:
            raise InvalidProxyScheme(payload["Message"], response.status_code, response.text)
        if error_code == 103:
            raise UnsupportedDriver(payload["Message"], response.status_code, response.text)
        if error_code == 104:
            raise UnsupportedRequestMethod(payload["Message"], response.status_code, response.text)
        if error_code == 105:
            raise SessionExpired(payload["Message"], response.status_code, response.text)
        if error_code == 106:
            raise WindowHandlerNotFound(payload["Message"], response.status_code, response.text)
        if error_code == 107:
            raise SessionError(payload["Message"], response.status_code, response.text)
        if error_code == 108:
            raise SessionNotFound(payload["Message"], response.status_code, response.text)
        if error_code == 109:
            raise Unauthorized(payload["Message"], response.status_code, response.text)
        if error_code == 110:
            raise TooManySessions(payload["Message"], response.status_code, response.text)
        if error_code == 111:
            raise ResourceNotFound(payload["Message"], response.status_code, response.text)
        if error_code == 112:
            raise JavascriptExecutionError(payload["Message"], response.status_code, response.text)
        if error_code == 113:
            raise MissingParameter(payload["Message"], response.status_code, response.text)
        if error_code == 114:
            raise InvalidSearchValue(payload["Message"], response.status_code, response.text)
        if error_code == 115:
            raise DriverDisabled(payload["Message"], response.status_code, response.text)
        if error_code == 116:
            raise ElementNotFound(payload["Message"], response.status_code, response.text)
        raise Exception(response.text)

    def send_request(self, command, parameters, require_auth=False):
        """
        Sends a request to the remote server
        向远程服务器发送请求

        :param command:
        :param parameters:
        :param require_auth:
        :return Response:
        """
        if require_auth:
            if self.authentication is not None:
                parameters["auth"] = self.authentication
        if command is None:
            response = requests.post(
                "{0}/".format(self.endpoint), parameters
            )
        else:
            response = requests.post(
                "{0}/{1}".format(self.endpoint, command), parameters
            )

        if response.status_code != 200:
            self.parse_error(response)
        return json.loads(response.text)



class Window(object):

    def __init__(self, data=None):
        """
        Public Constructor
        公共构造函数

        :param data:
        """
        if data is not None:
            self.id = data["ID"]
            self.url = data["Url"]
            self.title = data["Title"]



from netlenium.client import Client
from netlenium.proxy import Proxy
from netlenium.types import ProxyScheme, DriverType
from netlenium.window import Window


class Session(object):

    def __init__(self, data=None):
        """
        Public Constructor
        公共构造函数

        :param data:
        """

        if data is not None:
            self.id = data["ID"]
            self.created = data["Created"]
            self.last_activity = data["LastActivity"]
            self.driver = data["Driver"]
            self.headless = data["Headless"]

            scheme = None
            if data["ProxyConfiguration"]["Scheme"] == "http":
                scheme = ProxyScheme.http
            if data["ProxyConfiguration"]["Scheme"] == "https":
                scheme = ProxyScheme.https
            self.proxy_configuration = Proxy(
                host=data["ProxyConfiguration"]["Host"],
                port=data["ProxyConfiguration"]["Port"],
                scheme=scheme,
                authentication_required=data["ProxyConfiguration"]["AuthenticationRequired"],
                username=data["ProxyConfiguration"]["Username"],
                password=data["ProxyConfiguration"]["Password"]
            )
            self.proxy_configuration.enabled = data["ProxyConfiguration"]["Enabled"]

            self.current_window = Window()
            self.current_window.id = data["CurrentWindow"]["ID"]
            self.current_window.title = data["CurrentWindow"]["Title"]
            self.current_window.url = data["CurrentWindow"]["Url"]

    def create_client(self, admin_client, authentication=None):
        """
        Creates a client from this session
        从此会话创建客户端

        :type authentication: str
        :param authentication:
        :type admin_client: AdminClient
        :param admin_client:
        :return:
        """

        client = Client()

        if self.proxy_configuration.enabled:
            client.set_proxy_configuration(self.proxy_configuration)

        client.session_id = self.id
        client.endpoint = admin_client.endpoint

        if authentication is not None:
            client.api.authentication = authentication

        if self.driver == "chrome":
            client.target_driver = DriverType.chrome
        if self.driver == "opera":
            client.target_driver = DriverType.opera
        if self.driver == "firefox":
            client.target_driver = DriverType.firefox

        return client



from netlenium.api import API
from netlenium.element import Element
from netlenium.exceptions import ElementNotFound
from netlenium.exceptions import SessionError
from netlenium.types import DriverType
from netlenium.window import Window


class Client(object):

    def __init__(self, endpoint="http://localhost:6410", target_driver=DriverType.auto, authentication=None):
        """
        Public Constructor
        公共构造函数

        :param endpoint:
        :param target_driver:
        :param authentication:
        """
        self.endpoint = endpoint
        self.target_driver = target_driver
        self.proxy = None
        self.api = API(endpoint, authentication)
        self.session_id = None

    def set_proxy_configuration(self, proxy):
        """
        Sets a proxy configuration, this will be applied when starting a new session
        设置代理配置，这将在启动新会话时应用

        :param proxy:
        :return:
        """
        self.proxy = proxy

    def check_session(self):
        """
        Checks if the session is running, throws an exception if it's not
        检查会话是否正在运行，如果不是，则抛出异常

        :return:
        """
        if self.session_id is None:
            raise SessionError("The session is not running", 0)

    def start(self):
        """
        Starts the session and returns the Session ID
        启动会话并返回会话ID

        :return str:
        """
        parameters = {}

        if self.target_driver != DriverType.auto:
            parameters["target_driver"] = self.target_driver.name.lower()

        if self.proxy is not None:
            parameters["proxy_host"] = self.proxy.host
            parameters["proxy_port"] = self.proxy.port
            parameters["proxy_scheme"] = self.proxy.scheme.name.lower()
            if self.proxy.authentication_required:
                parameters["proxy_username"] = self.proxy.username
                parameters["proxy_password"] = self.proxy.password

        response = self.api.send_request("sessions/create", parameters)
        self.session_id = response["SessionId"]
        return self.session_id

    def stop(self):
        """
        Stops the current session and closes the driver
        停止当前会话并关闭驱动程序

        :return bool:
        """
        self.check_session()
        self.api.send_request("sessions/close",
                              {
                                  "session_id": self.session_id
                              }, True
                              )
        self.session_id = None
        return True

    def load_url(self, url):
        """
        Loads the URL, returns True when the navigation is completed
        加载URL，导航完成后返回True

        :param url:
        :return bool:
        """
        self.check_session()
        self.api.send_request("navigation/load_url",
                              {
                                  "session_id": self.session_id,
                                  "url": url
                              }, True
                              )
        return True

    def go_back(self):
        """
        Navigates one step backwards in the history, returns true when completed.
        在历史记录中向后导航一步，完成后返回true。

        :return bool:
        """
        self.check_session()
        self.api.send_request("navigation/go_back",
                              {
                                  "session_id": self.session_id
                              }, True
                              )
        return True

    def go_forward(self):
        """
        Navigates one stop forward in the history, returns true when completed.
        在历史记录中向后导航一步，完成后返回真。

        :return bool:
        """
        self.check_session()
        self.api.send_request("navigation/go_forward",
                              {
                                  "session_id": self.session_id
                              }, True
                              )
        return True

    def reload(self):
        """
        Reloads the current document
        重新加载当前文档

        :return bool:
        """
        self.check_session()
        self.api.send_request("navigation/reload",
                              {
                                  "session_id": self.session_id
                              }, True
                              )
        return True

    def get_elements(self, by, value):
        """
        Returns a list of element objects
        返回元素对象列表

        :param by:
        :param value:
        :return List[Element]:
        """
        self.check_session()
        response = self.api.send_request("actions/get_elements",
                                         {
                                             "session_id": self.session_id,
                                             "by": by.name.lower(),
                                             "value": value
                                         }, True
                                         )

        element_results = []
        for element_index, element in enumerate(response["Elements"]):
            element_results.append(Element(self, by.name.lower(), value, element_index, element))

        return element_results

    def get_element(self, by, value):
        """
        Returns the first occurrence of the element
        返回元素的第一个匹配项

        :param by:
        :param value:
        :return Element:
        """
        results = self.get_elements(by, value)
        if len(results) == 0:
            raise ElementNotFound("The element was not found", 0, None)
        return results[0]

    def execute_javascript(self, code):
        """
        Executes the given Javascript code in the current window of the session
        在会话的当前窗口中执行给定的Javascript代码

        :param code:
        :return:
        """
        self.check_session()
        response = self.api.send_request("actions/execute_javascript",
                                         {
                                             "session_id": self.session_id,
                                             "code": code
                                         }, True
                                         )
        return response["Output"]

    def current_window(self):
        """
        Returns the current window that the session is currently focused on
        返回会话当前关注的当前窗口

        :return Window:
        """
        self.check_session()
        response = self.api.send_request("window_handler/current_window",
                                         {
                                             "session_id": self.session_id
                                         }, True
                                         )
        return Window(response["CurrentWindow"])

    def list_windows(self):
        """
        Returns a list of Window handler id's that are currently opened
        返回当前打开的Window处理程序ID的列表

        :return:
        """
        self.check_session()
        response = self.api.send_request("window_handler/list_windows",
                                         {
                                             "session_id": self.session_id
                                         }, True
                                         )
        return response["WindowHandles"]

    def switch_window(self, window_id):
        """
        Switches to the Window Handle using the given ID
        使用给定的ID切换到窗口句柄

        :param window_id:
        :return:
        """
        self.check_session()
        self.api.send_request("window_handler/switch_to",
                              {
                                  "session_id": self.session_id,
                                  "id": window_id
                              }, True
                              )
        return True



class NetleniumException(Exception):
    def __init__(self, message, status_code, content=None):
        """
        Generic Netlenium Exception
        通用Netlenium异常

        :param message:
        :param status_code:
        :param content:
        """
        super(NetleniumException, self).__init__(message)
        self.status_code = status_code
        self.error_code = 1
        self.content = content


class AttributeNotFound(Exception):
    def __init__(self, message, status_code, content=None):
        """
        Exception thrown when the attribute value for the element was not found
        未找到元素的属性值时抛出异常

        :param message:
        :param status_code:
        :param content:
        """
        super(AttributeNotFound, self).__init__(message)
        self.status_code = status_code
        self.error_code = 101
        self.content = content


class InvalidProxyScheme(Exception):
    def __init__(self, message, status_code, content=None):
        """
        The given proxy scheme is invalid
        给定的代理方案无效

        :param message:
        :param status_code:
        :param content:
        """
        super(InvalidProxyScheme, self).__init__(message)
        self.status_code = status_code
        self.error_code = 102
        self.content = content


class UnsupportedDriver(Exception):
    def __init__(self, message, status_code, content=None):
        """
        Thrown when the given driver is unsupported on the remote server (New version maybe?)
        在远程服务器上不支持给定驱动程序时抛出（可能是新版本？）

        :param message:
        :param status_code:
        :param content:
        """
        super(UnsupportedDriver, self).__init__(message)
        self.status_code = status_code
        self.error_code = 103
        self.content = content


class UnsupportedRequestMethod(Exception):
    def __init__(self, message, status_code, content=None):
        """
        The request method used is not supported
        不支持使用的请求方法

        :param message:
        :param status_code:
        :param content:
        """
        super(UnsupportedRequestMethod, self).__init__(message)
        self.status_code = status_code
        self.error_code = 104
        self.content = content


class SessionExpired(Exception):
    def __init__(self, message, status_code, content=None):
        """
        The session has expired and is no longer available
        会话已过期，不再可用

        :param message:
        :param status_code:
        :param content:
        """
        super(SessionExpired, self).__init__(message)
        self.status_code = status_code
        self.error_code = 105
        self.content = content


class WindowHandlerNotFound(Exception):
    def __init__(self, message, status_code, content=None):
        """
        The given Window Handler was not found in the session
        在会话中找不到给定的窗口处理程序

        :param message:
        :param status_code:
        :param content:
        """
        super(WindowHandlerNotFound, self).__init__(message)
        self.status_code = status_code
        self.error_code = 106
        self.content = content


class SessionError(Exception):
    def __init__(self, message, status_code, content=None):
        """
        There was a unexpected error with the session
        会话出现意外错误

        :param message:
        :param status_code:
        :param content:
        """
        super(SessionError, self).__init__(message)
        self.status_code = status_code
        self.error_code = 107
        self.content = content


class SessionNotFound(Exception):
    def __init__(self, message, status_code, content=None):
        """
        The session was not found on the server
        在服务器上找不到该会话

        :param message:
        :param status_code:
        :param content:
        """
        super(SessionNotFound, self).__init__(message)
        self.status_code = status_code
        self.error_code = 108
        self.content = content


class Unauthorized(Exception):
    def __init__(self, message, status_code, content=None):
        """
        You don't have authorized access to make this request.
        您没有授权访问权来发出此请求。

        :param message:
        :param status_code:
        :param content:
        """
        super(Unauthorized, self).__init__(message)
        self.status_code = status_code
        self.error_code = 109
        self.content = content


class TooManySessions(Exception):
    def __init__(self, message, status_code, content=None):
        """
        The server cannot handle anymore sessions
        服务器无法再处理会话

        :param message:
        :param status_code:
        :param content:
        """
        super(TooManySessions, self).__init__(message)
        self.status_code = status_code
        self.error_code = 110
        self.content = content


class ResourceNotFound(Exception):
    def __init__(self, message, status_code, content=None):
        """
        The requested resource was not found
        找不到请求的资源

        :param message:
        :param status_code:
        :param content:
        """
        super(ResourceNotFound, self).__init__(message)
        self.status_code = status_code
        self.error_code = 111
        self.content = content


class JavascriptExecutionError(Exception):
    def __init__(self, message, status_code, content=None):
        """
        The given JavaScript code failed to be executed
        给定的JavaScript代码无法执行

        :param message:
        :param status_code:
        :param content:
        """
        super(JavascriptExecutionError, self).__init__(message)
        self.status_code = status_code
        self.error_code = 112
        self.content = content


class MissingParameter(Exception):
    def __init__(self, message, status_code, content=None):
        """
        The request is missing a required parameter
        请求缺少必需参数

        :param message:
        :param status_code:
        :param content:
        """
        super(MissingParameter, self).__init__(message)
        self.status_code = status_code
        self.error_code = 113
        self.content = content


class InvalidSearchValue(Exception):
    def __init__(self, message, status_code, content=None):
        """
        The given search value for finding elements is not supported/is invalid
        查找元素的给定搜索值不受支持/无效

        :param message:
        :param status_code:
        :param content:
        """
        super(InvalidSearchValue, self).__init__(message)
        self.status_code = status_code
        self.error_code = 114
        self.content = content


class DriverDisabled(Exception):
    def __init__(self, message, status_code, content=None):
        """
        The administrator has disabled support for this driver
        管理员已禁用对此驱动程序的支持

        :param message:
        :param status_code:
        :param content:
        """
        super(DriverDisabled, self).__init__(message)
        self.status_code = status_code
        self.error_code = 115
        self.content = content


class ElementNotFound(Exception):
    def __init__(self, message, status_code, content=None):
        """
        The element was not found
        找不到该元素

        :param message:
        :param status_code:
        :param content:
        """
        super(ElementNotFound, self).__init__(message)
        self.status_code = status_code
        self.error_code = 116
        self.content = content



from enum import Enum


class DriverType(Enum):
    """
    Supported Driver Type
    支持的驱动程序
    """
    auto = 1
    chrome = 2
    firefox = 3
    opera = 4


class ProxyScheme(Enum):
    """
    Supported Proxy Scheme
    支持的代理方案
    """
    http = 1
    https = 2


class By(Enum):
    """
    Search elements by value
    按值搜索元素
    """
    class_name = 1
    css_selector = 2
    id = 3
    name = 4
    tag_name = 5
    xpath = 6



from netlenium.admin_client import AdminClient
from netlenium.api import API
from netlenium.client import Client
from netlenium.exceptions import *
from netlenium.types import DriverType
from netlenium.types import ProxyScheme
from netlenium.window import Window
from netlenium.proxy import Proxy

__all__ = [
    "API",
    "Window",
    "DriverType",
    "Proxy"
    "ProxyScheme",
    "Client",
    "AdminClient",
    "NetleniumException",
    "AttributeNotFound",
    "InvalidProxyScheme",
    "UnsupportedDriver",
    "UnsupportedRequestMethod",
    "SessionExpired",
    "WindowHandlerNotFound",
    "SessionError",
    "SessionNotFound",
    "Unauthorized",
    "TooManySessions",
    "ResourceNotFound",
    "JavascriptExecutionError",
    "MissingParameter",
    "InvalidSearchValue",
    "DriverDisabled",
    "ElementNotFound"
]



from netlenium.api import API
from netlenium.session import Session


class AdminClient(object):

    def __init__(self, endpoint="http://localhost:6410", authentication=None):
        """
        Public Constructor
        公共构造函数

        :param endpoint:
        :param authentication:
        """
        self.endpoint = endpoint
        self.api = API(endpoint, authentication)

    def get_sessions(self):
        response = self.api.send_request("admin/active_sessions", {}, True)

        session_results = []
        for session in response["Sessions"]:
            session_results.append(Session(session))

        return session_results



from netlenium.types import ProxyScheme


class Proxy(object):

    def __init__(self, host, port,
                 scheme=ProxyScheme.https, authentication_required=False, username=None, password=None
                 ):
        """
        Proxy Configuration
        代理配置

        :param host:
        :param port:
        :param scheme:
        :param authentication_required:
        :param username:
        :param password:
        """
        self.host = host
        self.port = port
        self.scheme = scheme
        self.authentication_required = authentication_required
        self.username = username
        self.password = password
        self.enabled = True



from setuptools import setup

setup(name='netlenium',
      version='1.0.0.1',
      description='Automate Web Browsers such as Chrome, Opera and Firefox  using Netlenium',
      url='https://github.com/intellivoid/Netlenium-Python-Wrapper',
      author='Intellivoid',
      author_email='narrakas@intellivoid.info',
      packages=['netlenium'],
      zip_safe=False
      )



class Element(object):

    def __init__(self, client, by, value, index, data=None):
        """
        Public Constructor
        公共构造函数

        :param client:
        :param by:
        :param value:
        :param index:
        :param data:
        """

        self.client = client
        self.by = by
        self.value = value
        self.index = index
        if data is not None:
            self.enabled = data["Enabled"]
            self.is_selected = data["IsSelected"]
            self.location = {
                "x": data["ElementLocation"]["X"],
                "y": data["ElementLocation"]["Y"]
            }
            self.size = {
                "width": data["ElementSize"]["Width"],
                "Height": data["ElementSize"]["Height"]
            }
            self.tag_name = data["TagName"],
            self.inner_text = data["InnerText"]

    def send_keys(self, keys_input):
        """
        Simulates typing into the element using the given input
        使用给定输入模拟键入元素

        :param keys_input:
        :return:
        """

        self.client.check_session()
        self.client.api.send_request("web_element/send_keys",
                                     {
                                         "session_id": self.client.session_id,
                                         "by": self.by,
                                         "value": self.value,
                                         "index": str(self.index),
                                         "input": str(keys_input)
                                     }
                                     )
        return True

    def click(self):
        """
        Simulates a click event on the element
        模拟元素上的单击事件

        :return bool:
        """

        self.client.check_session()
        self.client.api.send_request("web_element/click",
                                     {
                                         "session_id": self.client.session_id,
                                         "by": self.by,
                                         "value": self.value,
                                         "index": str(self.index)
                                     }
                                     )
        return True

    def clear(self):
        """
        Clears the element's inner content
        清除元素的内部内容

        :return:
        """

        self.client.check_session()
        self.client.api.send_request("web_element/clear",
                                     {
                                         "session_id": self.client.session_id,
                                         "by": self.by,
                                         "value": self.value,
                                         "index": self.index
                                     }
                                     )
        return True

    def move_to(self):
        """
        Moves into the element's view
        移动到元素的视图中

        :return:
        """
        self.client.check_session()
        self.client.api.send_request("web_element/move_to",
                                     {
                                         "session_id": self.client.session_id,
                                         "by": self.by,
                                         "value": self.value,
                                         "index": self.index
                                     }
                                     )
        return True

    def submit(self):
        """
        Submits the element to the remote web server
        将元素提交到远程Web服务器

        :return:
        """
        self.client.check_session()
        self.client.api.send_request("web_element/submit",
                                     {
                                         "session_id": self.client.session_id,
                                         "by": self.by,
                                         "value": self.value,
                                         "index": self.index
                                     }
                                     )
        return True

    def get_attribute(self, attribute_name):
        """
        Get's the element's value for an attribute
        获取属性的元素值

        :param attribute_name:
        :return:
        """
        self.client.check_session()
        response = self.client.api.send_request("web_element/get_attribute",
                                                {
                                                    "session_id": self.client.session_id,
                                                    "by": self.by,
                                                    "value": self.value,
                                                    "index": self.index,
                                                    "attribute_name": attribute_name
                                                }
                                                )
        return response["AttributeValue"]

    def set_attribute(self, attribute_name, attribute_value):
        """
        Sets a value to the given element's attribute, if the attribute does not exist it will be created
        将值设置为给定元素的属性，如果该属性不存在，则将创建该属性

        :param attribute_name:
        :param attribute_value:
        :return:
        """
        self.client.check_session()
        self.client.api.send_request("web_element/set_attribute",
                                     {
                                         "session_id": self.client.session_id,
                                         "by": self.by,
                                         "value": self.value,
                                         "index": self.index,
                                         "attribute_name": attribute_name,
                                         "attribute_value": attribute_value
                                     }
                                     )
        return True
